// $Id: command-log.hc,v 1.3 2003/11/22 00:16:43 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lib/fmt.h"

static void
logCommand(struct Parameters *params)
{
  Logging_timestamp(2); WRITE_MSG(2, "--- mark ---\n");
  Logging_timestamp(1);

  WRITE_MSG(1, ">>> executing [");

  char const	*delim="'";
  
  for (size_t i=0; i<params->argc; ++i) {
    WRITE_STR(1, delim);
    WRITE_STR(1, params->argv[i]);
    WRITE_MSG(1, "'");

    delim = ", '";
  }

  WRITE_MSG(1, "]\n");
}

static void
logProcessResult(int pid, int status)
{
  char		buf[sizeof(int)*4 + 2];
  size_t	l = fmt_long(buf, pid);
  bool		is_ok = false;
  
  Logging_timestamp(1);
  
  WRITE_MSG  (1, "process ");
  (void)write(1, buf, l);
  WRITE_MSG  (1, " exited with status 0x");
  
  l = fmt_long_base(buf, status, 16);
  (void)write(1, buf, l);
  WRITE_MSG  (1, " (");

  if (WIFEXITED(status)) {
    l = fmt_long(buf, WEXITSTATUS(status));
    WRITE_MSG  (1, "exitcode ");
    (void)write(1, buf, l);

    is_ok = WEXITSTATUS(status)==0;
  }
  else if (WIFSIGNALED(status)) {
    l = fmt_long(buf, WTERMSIG(status));
    WRITE_MSG  (1, "killed by signal ");
    (void)write(1, buf, l);
  }
  else if (WIFSTOPPED(status)) {
    l = fmt_long(buf, WSTOPSIG(status));
    WRITE_MSG  (1, "stopped by signal ");
    (void)write(1, buf, l);
  }
  else WRITE_MSG(1, "???");

  WRITE_MSG(1, ")\n");

  if (is_ok)
  { Logging_timestamp(2); WRITE_MSG(2, "--- mark ---\n"); }
  else {
    Logging_timestamp(2); WRITE_MSG(2, "!!! failed with status 0x");
    
    l = fmt_long_base(buf, status, 16);
    (void)write(2, buf, l);
    WRITE_MSG  (2, " !!!\n");
  }
}

static void
logCheckResult(unsigned int code, char const *reason)
{
  char			buf[sizeof(unsigned int)*4 + 1];
  size_t		l = fmt_ulong(buf, code);
    
  Logging_timestamp(2);
  WRITE_MSG(2, "parameters failed to verify: ");
  WRITE_STR(2, reason);
  WRITE_MSG(2, "' (code ");
  write    (2, buf, l);
  WRITE_MSG(2, "'\n");
}
