// $Id: djinni-rub.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "djinni.h"
#include "wrappers.h"
#include "util-io.h"
#include "msg-strings.h"

#include <libgen.h>
#include <sys/un.h>
#include <getopt.h>
#include <assert.h>

int	wrapper_exit_code = 1;

struct ArgInfo
{
    bool		do_fork;
    char const *	pid_file;
    char const *	log_file;
    char const *	err_file;
    char const *	sockname;
};

#include "daemonize.hc"

static struct option const
CMDLINE_OPTIONS[] = {
  { "help",     no_argument,  0, 'h' },
  { "version",  no_argument,  0, 'v' },
  { "daemon",   no_argument,  0, 'd' },
  { 0,0,0,0 }
};

static void
showHelp(int fd, char *cmd, int res)
{
  WRITE_MSG(fd, "Usage:  ");
  WRITE_STR(fd, basename(cmd));
  WRITE_MSG(fd,
	    " [--help] [--version] [--daemon|-d] [--] [<sockname>]\n\n"
	    "Please report bugs to " PACKAGE_BUGREPORT "\n");
  exit(res);
}

static void
showVersion()
{
  WRITE_MSG(1,
	    "djinni-rub " VERSION " -- opens the lamp\n"
	    "This program is part of " PACKAGE_STRING "\n\n"
	    "Copyright (C) 2003 Enrico Scholz\n"
	    VERSION_COPYRIGHT_DISCLAIMER);
  exit(0);
}

static void
getArguments(struct ArgInfo *args, int argc, char *argv[])
{
  while (1) {
    int		c = getopt_long(argc, argv, "d", CMDLINE_OPTIONS, 0);

    if (c==-1) break;

    switch (c) {
      case 'h'	:  showHelp(1, argv[0], 0);
      case 'v'	:  showVersion();
      case 'd'	:  args->do_fork = true; break;
      default	:
	WRITE_MSG(2, "Try '");
	WRITE_STR(2, argv[0]);
	WRITE_MSG(2, " --help\" for more information.\n");
	exit(1);
	break;
    }
  }

  if (optind+1<argc) {
    WRITE_MSG(2, "Unknown argument, please try '--help' for more information\n");
    exit(1);
  }

  if (optind<argc) args->sockname = argv[optind];  
}  

int main(int argc, char *argv[])
{
  struct ArgInfo	args = {
    .do_fork  = false,
    .pid_file = DJINNI_RUB_PIDFILE,
    .log_file = "/dev/null",
    .err_file = "/dev/null",
    .sockname = DJINNI_HEARTBEAT_SOCKET,
  };
  
  getArguments(&args, argc, argv);

  struct sockaddr_un	addr = {
    .sun_family = AF_UNIX,
  };

  strncpy(addr.sun_path, args.sockname, sizeof(addr.sun_path));
  
  int		fd = Esocket(PF_UNIX, SOCK_STREAM, 0);
  Econnect(fd, &addr, sizeof addr);

  char		c;
  if (!EreadAll(fd, &c, sizeof c)) {
    WRITE_MSG(2, "unexpected synchronization event\n");
    return EXIT_FAILURE;
  }

  if (args.do_fork) {
    int			stat_fd = daemonize(&args);
    pid_t		pid     = getpid();

    EwriteAll(stat_fd, &pid, sizeof pid);
    Eclose(stat_fd);
  }
  
  while (EreadAll(fd, &c, sizeof c)) {}
  return EXIT_SUCCESS;
}
