// $Id: djinni.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "wish.h"
#include "msg-strings.h"
#include "util-io.h"
#include "util.h"
#include "djinni.h"

#include <libgen.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>

#ifndef UNIX_PATH_MAX
#  define UNIX_PATH_MAX    108
#endif

struct ArgInfo
{
    char const *		sockname;
    int				argc;
    char const * const *	argv;
    bool			is_quiet;
};

int	wrapper_exit_code = 255;

static struct option const
CMDLINE_OPTIONS[] = {
  { "help",     no_argument,  0, 'h' },
  { "version",  no_argument,  0, 'v' },
  { "socket",   required_argument, 0, 's' },
  { 0,0,0,0 }
};

static void
showHelp(int fd, char *cmd, int res)
{
  WRITE_MSG(fd, "Usage:  ");
  WRITE_STR(fd, basename(cmd));
  WRITE_MSG(fd,
	    " [--help] [--version] [--socket|-s <filename>] [-q] [--] <command> <args>*\n\n"
	    "Please report bugs to " PACKAGE_BUGREPORT "\n");
  exit(res);
}

static void
showVersion()
{
  WRITE_MSG(1,
	    "djinni " VERSION " -- sends a command to the djinni daemon\n"
	    "This program is part of " PACKAGE_STRING "\n\n"
	    "Copyright (C) 2003 Enrico Scholz\n"
	    VERSION_COPYRIGHT_DISCLAIMER);
  exit(0);
}

static int
doit(struct ArgInfo const *args)
{
  int			fd;
  struct Wish		wish;
  struct WishResult	res;
  struct sockaddr_un	addr = {
    .sun_family = AF_UNIX
  };

  strcpy(addr.sun_path, args->sockname);
    
  fd = Esocket(PF_UNIX, SOCK_STREAM, 0);
  Econnect(fd, &addr, sizeof addr);

  Wish_init(&wish, fd, 0);
  res = Wish_send(&wish, args->argc, args->argv);

  if (res.res!=resOK) {
    WRITE_MSG(2, "Communication error with djinni daemon\n");
    exit(255);
  }

  if (res.len>0 && (res.code!=0 || !args->is_quiet)) {
    int	out_fd = (res.code==0) ? 1 : 2;
    (void)write(out_fd, res.data, res.len);
    WRITE_MSG(out_fd, "\n");
  }
  Eclose(fd);

  return res.code;
}

int main(int argc, char *argv[])
{
  struct ArgInfo	args = {
    .sockname = DJINNI_CMD_SOCKET,
    .is_quiet = false,
  };

  while (1) {
    int		c = getopt_long(argc, argv, "hvqs:", CMDLINE_OPTIONS, 0);

    if (c==-1) break;

    switch (c) {
      case 'h'	:  showHelp(1, argv[0], 0);
      case 'v'	:  showVersion();
      case 's'	:  args.sockname = optarg; break;
      case 'q'	:  args.is_quiet = true;   break;
      default	:
	WRITE_MSG(2, "Try '");
	WRITE_STR(2, argv[0]);
	WRITE_MSG(2, " --help\" for more information.\n");
	return EXIT_FAILURE;
	break;
    }
  }

  if (optind==argc) {
    WRITE_MSG(2, "No command specified\n");
    return EXIT_FAILURE;
  }

  if (strlen(args.sockname)>=UNIX_PATH_MAX) {
    WRITE_MSG(2, "Socket-filename too long\n");
    return EXIT_FAILURE;
  }

  args.argc = argc-optind;
  args.argv = const_cast(char const * const *)(argv)+optind;

  return doit(&args);
}
