// $Id: djinniprocess-new.c,v 1.2 2003/11/07 00:44:35 ensc Exp $    --*- c -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "djinniprocess.h"
#include "djinniprocess-internal.h"
#include "wrappers.h"
#include "util-io.h"
#include "djinni.h"
#include "exithandler.h"

#include <sys/un.h>

static void
cleanupStateDir(void *djinni_v)
{
  struct DjinniProcess		*djinni = djinni_v;
  DjinniProcess_delete(djinni);
}


static void
initDirFDs(struct DjinniProcess *self)
{
  self->dirfd_state = Eopen(".", O_RDONLY, 0);
  self->dirfd_root  = Eopen("/", O_RDONLY, 0);

  setCloseOnExec(self->dirfd_state);
  setCloseOnExec(self->dirfd_root);
  setCloseOnExec(self->dirfd_vserver);
}

static void
populateStateDir(struct DjinniProcess *self)
{
  bool			is_parent_trusted = true;
  bool			is_trusted;

  if (self->parent==0){
    Etouch(".root");
  }
  else {
    is_parent_trusted = self->parent->is_trusted;
  }

  is_trusted = access(".cfg/.trusted", F_OK)!=-1;
  if (!is_parent_trusted && !is_trusted) {
    WRITE_MSG(2, "Can not operate in untrusted vservers with untrusted parents\n");
    exit(2);
  }

  self->is_trusted = is_parent_trusted && is_trusted;
  if (self->is_trusted) Etouch(".trusted");

  struct sockaddr_un	addr = {
    .sun_family = AF_UNIX,
    .sun_path   = ".cmd"
  };
  
  self->fd_cmd_master = Esocket(PF_UNIX, SOCK_STREAM, 0);
  Ebind(self->fd_cmd_master, &addr, sizeof addr);
  setCloseOnExec(self->fd_cmd_master);
}

static void
createVserverSockets(struct DjinniProcess *self)
{
  struct sockaddr_un	addr_cmd  = {
    .sun_family = AF_UNIX,
    .sun_path   = DJINNI_CMD_SOCKET
  };
  struct sockaddr_un	addr_beat = {
    .sun_family = AF_UNIX,
    .sun_path   = DJINNI_HEARTBEAT_SOCKET
  };

  Efchdir(self->dirfd_vserver);
  Echroot(".");
  
  unlink(DJINNI_CMD_SOCKET);
  unlink(DJINNI_HEARTBEAT_SOCKET);

  self->fd_cmd_vserver       = Esocket(PF_UNIX, SOCK_STREAM, 0);
  Ebind(self->fd_cmd_vserver, &addr_cmd, sizeof addr_cmd);
  setCloseOnExec(self->fd_cmd_vserver);

  if (self->parent!=0) {
    self->fd_heartbeat_vserver = Esocket(PF_UNIX, SOCK_STREAM, 0);
    Ebind(self->fd_heartbeat_vserver, &addr_beat, sizeof addr_beat);
    setCloseOnExec(self->fd_heartbeat_vserver);
  }

  if (!DjinniProcess_chdirState(self)) {
    perror("DjinniProcess_restoreCurrentDir()");
    exit(1);
  }
}

static void
setEnvironment(struct DjinniProcess const *self)
{
  Esetenv("DJINNI_STATEDIR", self->state_dir, 1);
  
  if (self->is_trusted) Esetenv("DJINNI_TRUSTED",  "1", 1);
  else                  Eunsetenv("DJINNI_TRUSTED");

  if (self->parent==0)  Esetenv("DJINNI_ROOT", "1", 1);
  else                  Eunsetenv("DJINNI_ROOT");
}

struct DjinniProcess *
DjinniProcess_new(int dirfd_vserver,
		  char const *cfg_dir,
		  char const *state_dir,
		  struct DjinniProcess const *parent,
		  int heartbeat_fd)
{  
  struct DjinniProcess	*self = Emalloc(sizeof(struct DjinniProcess));
    
  Epipe(self->fd_heartbeat_children);
  setCloseOnExec(self->fd_heartbeat_children[0]);
  setCloseOnExec(self->fd_heartbeat_children[1]);

  if (parent!=0)
    Emkdir(state_dir, 0600);
  Echdir(state_dir);
  
  self->state_dir            = get_current_dir_name();
  self->dirfd_vserver	     = dirfd_vserver;
  self->parent               = parent;
  self->fd_parent	     = parent ? parent->fd_heartbeat_children[0] : heartbeat_fd;
  self->fd_cmd_vserver       = -1;
  self->fd_cmd_master	     = -1;
  self->fd_heartbeat_vserver = -1;
  self->dirfd_root           = -1;
  self->dirfd_state          = -1;
  self->is_alive	     = true;
  self->credentials.f        = false;

  initDirFDs(self);
  
  ExitHandler_addHandler(cleanupStateDir, self);

  Esymlink(cfg_dir, ".cfg");
  populateStateDir(self);
  createVserverSockets(self);

  setEnvironment(self);
  
  return self;
}
