// $Id: getctxproc.c,v 1.2 2003/11/06 04:54:43 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "getctxproc.h"
#include "util-io.h"
#include "wrappers.h"
#include "wrappers-io.h"
#include "logging.h"

#include <assert.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <vserver.h>

  //#define FAKE_VC_FUNCS	1

#ifndef S_CTX_INFO_LOCK
#  define S_CTX_INFO_LOCK	2
#endif

typedef pid_t	QueryType;
typedef ctx_t	ResultType;

static bool	run(struct GetContextProcess *self) NONNULL(1);
static bool	initServer(struct GetContextProcess *self,
			   int *comm_fd, int *sync_fd) NONNULL(1,2,3);
static void	initClient(struct GetContextProcess *self,
			   int *comm_fd, int *sync_fd) NONNULL(1,2,3);

static bool
initServer(struct GetContextProcess *self, int *comm_fd, int *sync_fd)
{
  assert(self!=0 && comm_fd!=0 && sync_fd!=0);

  char		buf[2];

  Eclose(comm_fd[0]);
  Eclose(sync_fd[1]);
  self->fd_   = comm_fd[1];
  self->sync_ = sync_fd[0];

  if (Eread(self->fd_, buf, 2)!=2 ||
      memcmp(buf, "OK", 2)!=0) {
    close(self->fd_);
    return false;
  }
  
  return true;
}

static void
initClient(struct GetContextProcess *self, int *comm_fd, int *sync_fd)
{
  assert(self!=0 && comm_fd!=0 && sync_fd!=0);

  Eclose(comm_fd[1]);
  Eclose(sync_fd[0]);
  self->fd_   = comm_fd[0];
  self->sync_ = sync_fd[1];

#ifndef FAKE_VC_FUNCS
  if (vc_new_s_context(1, ~0, S_CTX_INFO_LOCK)==-1) {
    perror("vc_new_s_context()");
    exit(1);
  }
#endif
  
  if (Ewrite(self->fd_, "OK", 2)!=2)
    exit(1);
}

bool
GetContextProcess_init(struct GetContextProcess *self)
{
  assert(self!=0);

  int	fd[2];
  int	sync[2];

  Esocketpair(AF_UNIX, SOCK_STREAM, 0, fd);
  Epipe(sync);
  
  setCloseOnExec(fd[0]);
  setCloseOnExec(fd[1]);
  setCloseOnExec(sync[0]);
  setCloseOnExec(sync[1]);
  
  self->pid_ = Efork();

  if (self->pid_!=0)
    return initServer(self, fd, sync);
  else {
#ifndef TESTSUITE
    Logging_newProcess("@ctxproc");
#endif
    initClient(self, fd, sync);
    if (!run(self)) exit(1);
    exit(0);
  }
}

void
GetContextProcess_detach(struct GetContextProcess *self)
{
  close(self->sync_);
  close(self->fd_);
}

void
GetContextProcess_delete(struct GetContextProcess *self)
{
  assert(self!=0);

  GetContextProcess_detach(self);
}

static bool
run(struct GetContextProcess *self)
{
  assert(self!=0);


  for (;;) {
    QueryType		query;
    size_t		len;

    len = Ewrite(self->sync_, ".", 1);
    if (len==0) return true;

    len = Eread(self->fd_, &query, sizeof(query));
    if (len==0) return true;
    
    if (len!=sizeof(query)) {
      WRITE_MSG(2, "Unexpected request\n");
      return false;
    }

#ifndef FAKE_VC_FUNCS
    ResultType		result = vc_X_getctx(query);
#else
    ResultType		result = 0;
#endif

    len = write(self->fd_, &result, sizeof(result));
    if (len!=sizeof(result)) return false;
  }

  assert(false);
}

ctx_t
GetContextProcess_getctx(struct GetContextProcess * self,
			 pid_t pid)
{
  assert(self!=0);

  char		sync;
  if (!WreadAll(self->sync_, &sync, 1)) {
    WRITE_MSG(2, "Bad synchronization\n");
    exit(1);
  }

  QueryType		query = pid;
  if (!WwriteAll(self->fd_, &query, sizeof(query))) {
    WRITE_MSG(2, "Could not sent ctx-request\n");
    exit(1);
  }

  ResultType		result;
  if (!WreadAll(self->fd_, &result, sizeof(result))) {
    WRITE_MSG(2, "Could not receive result of ctx-request\n");
    exit(1);
  }

  return result;
}
