// $Id: wish-send.c,v 1.1 2003/11/04 22:32:40 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "wish.h"
#include "wish-internal.h"
#include "util-io.h"
#include "wrappers-io.h"

struct WishResult
Wish_send(struct Wish *self, size_t argc, char const * const argv[])
{
  struct WishResult	result = {
    .res  = resFAIL,
    .len  = 0,
    .code = 0,
    .data = 0
  };
    
  {
    Wish_ArgCountType	tmp = argc;
    if (tmp!=argc ||
	!WsendAll(self->fd_, &tmp, sizeof tmp))
      return result;
  }

  for (size_t i=0; i<argc; ++i) {
    Wish_ArgSizeType	tmp = strlen(argv[i]);

    if (tmp!=strlen(argv[i]) ||
	!WsendAll(self->fd_, &tmp, sizeof tmp) ||
	!WsendAll(self->fd_, argv[i], tmp))
      return result;
  }

    // Receive the 'PLEASE ACK' request and give the ACK
  {
    Wish_ArgCountType	cnt;
    Wish_ArgSizeType	len;
    char		buf[sizeof(WISH_PLEASE_ACK)];
    
    if (!WrecvAll(self->fd_, &cnt, sizeof cnt) ||
	cnt!=2 ||
	!WrecvAll(self->fd_, &len, sizeof len) ||
	len!=sizeof(buf) ||
	!WrecvAll(self->fd_, buf,  len) ||
	memcmp(buf, WISH_PLEASE_ACK, sizeof buf)!=0   ||
	!WrecvAll(self->fd_, &len, sizeof len) ||
	len!=sizeof(self->confirm_data_) ||
	!WrecvAll(self->fd_, self->confirm_data_, len) ||
	!WsendAll(self->fd_, &len, sizeof len) ||
	!WsendAll(self->fd_, self->confirm_data_, len))
      return result;
  }

    // Receive the result
  {
    Wish_ArgSizeType	len;
    uint16_t		code;

    if (!WrecvAll(self->fd_, &len,   sizeof len) ||
	len<sizeof(code) ||
	!WrecvAll(self->fd_, &code,  sizeof code))
      return result;

    len -= sizeof(code);

    if (len>0 &&
	((result.data=Emalloc(len))==0 ||
	 !WrecvAll(self->fd_, result.data, len)) ) {
      free(result.data);
      return result;
    }

    result.res  = resOK;
    result.len  = len;
    result.code = code;

    return result;
  }
}
